function [varargout] = rk4(vfun,vslot,vinit,odeoptions,varargin)
% 
% [] = rk4(@FUN,SLOT,INIT,ODEOPTIONS,[PAR1, PAR2, ...])
%
% [SOL] = rk4(@FUN,SLOT,INIT,ODEOPTIONS,[PAR1, PAR2, ...])
%
% [T, Y] = rk4(@FUN,SLOT,INIT,ODEOPTIONS,[PAR1, PAR2, ...])
%
% This function file can be used to solve a set of non-stiff
% ordinary differential equations (non-stiff ODEs) with the well
% known explicit Runge-Kutta method of order 4.
%
% If this function is called with no return argument then plot the
% solution over time in a figure window while solving the set of
% ODEs that are defined in a function and specified by the function
% handle @FUN. The second input argument SLOT is a double vector
% that defines the time slot, INIT is a double vector that defines
% the initial values of the states, OPT is a mandatory dummy argument 
% (not used) and PAR1, PAR2, ... can optionally be other input
% arguments of any type that have to be passed to the function
% defined by @FUN.
%
% If this function is called with two return arguments then
% return the time stamps T = SLOT and the solution values Y.

dt = diff(vslot);
v = zeros(length(vslot),length(vinit));
v(1,:) = vinit(:).';
for j = 1:length(dt)
  t = vslot(j);
  k1 = feval(vfun,t,v(j,:).',varargin{:});
  k2 = feval(vfun,t+dt(j)/2,v(j,:).'+k1*dt(j)/2,varargin{:});
  k3 = feval(vfun,t+dt(j)/2,v(j,:).'+k2*dt(j)/2,varargin{:});
  k4 = feval(vfun,t+dt(j),v(j,:).'+k3*dt(j),varargin{:});
  v(j+1,:) = v(j,:)+dt(j)*(k1.'+2*k2.'+2*k3.'+k4.')/6;
  if (~isempty(odeoptions.OutputFcn))
    v(j+1,:) = feval(odeoptions.OutputFcn,t,v(j+1,:).','init',varargin{:});
  end
  if (nargout == 0)
    drawnow
    plot(vslot(1:j+1),v(1:j+1,:))
  end
end
if (nargout == 1)
  varargout{1}.x = vslot(:);
  varargout{1}.y = v;
  varargout{1}.solver = 'rk4'; 
elseif (nargout == 2)
  varargout{1} = vslot(:);
  varargout{2} = v;
end